from __future__ import annotations
from pathlib import Path

try:
    # Version generated by setuptools_scm during build
    from ._version import __version__
except ImportError:
    __version__ = None

# Top-level API (available directly from "zut" namespace). Does not require any dependency.
from .choices import TextChoices, IntegerChoices, choices_table
from .colors import Colors
from .commands import CommandManager, CommandResource, ArgumentParser, add_func_command, add_module_command, add_package_commands, get_help_text, get_description_text, main, create_django_command
from .config import ConfigurationError, ExtendedConfigParser, get_env_config, get_secret_config, get_config_parser, parse_env_file, _UNSET
from .csv import get_csv_headers, get_csv_dict_list, load_from_csv, dump_to_csv
from .date import parse_tz, now_aware, make_aware, make_naive, is_aware, is_naive
from .db import DbAdapter, PgAdapter, Pg2Adapter, MssqlAdapter, get_db_adapter, get_db_adapter_with_schema_and_table, pg_notice_handler, Pg2NoticeHandler
from .format import Format, configure_format
from .files import configure_smb_credentials  # other functions should be called directly with the module, e.g. `from zut import files`, then `files.open(...)`
from .http import JSONApiClient, ApiClientError
from .inout import out_file, out_table, in_table, transfer_table, OutFile, OutTable, OutTabulate, OutCsv, OutExcel, OutDb, InTable, InCsv, InExcel, InDb, Row, normalize_inout, register_out_file_handler, register_out_table_handler, register_in_table_handler
from .json import ExtendedJSONDecoder, ExtendedJSONEncoder
from .logging import configure_logging, get_logging_dict_config
from .misc import topological_sort
from .numeric import as_gib, as_mib, as_kib, human_bytes, human_number
from .process import get_exit_code, check_completed_subprocess
from .text import Filters, Filter, ValueString, slugify, slugify_django, slugify_snake, remove_whitespaces, remove_consecutive_whitespaces, skip_utf8_bom, build_url, build_netloc, merge_ip_addresses, is_valid_uuid
from .types import get_iterable_element_type, is_iterable_of, convert, convert_to_bool, convert_to_int, convert_to_decimal, get_leaf_classes
from .unittest import StartTestProgram

ZUT_ROOT = Path(__file__).parent
""" Root directory of `zut` library """

# NOTE: Inner API: the following modules require extra dependencies.
# zut.django
# zut.excel
